import React, { useEffect, useState } from "react";

/**
 * ThemeSvg component that renders an SVG with theme colors
 *
 * @param {Object} props - Component props
 * @param {string} props.src - Source path to the SVG file
 * @param {number|string} props.width - Width of the SVG
 * @param {number|string} props.height - Height of the SVG
 * @param {string} props.className - CSS class for the SVG
 * @param {string} props.alt - Alt text for the SVG
 * @param {Object} props.colorMap - Optional mapping of original colors to CSS variables
 * @returns {React.ReactElement}
 */
const ThemeSvg = ({ src, width, height, className, alt, colorMap = {} }) => {
  const [svgContent, setSvgContent] = useState("");
  const [isLoading, setIsLoading] = useState(true);
  const [error, setError] = useState(null);

  useEffect(() => {
    if (!src) {
      setError("No source provided");
      setIsLoading(false);
      return;
    }

    // Default color mappings if not provided
    const defaultColorMap = {
      // Primary color gradients
      "#ef5488": "var(--primary-color)",
      "#6c245a": "var(--primary-color)",
      "#8f235c": "var(--primary-color)",
      "#80234e": "var(--primary-color)",
      "#8c2854": "var(--primary-color)",
      "#ad3665": "var(--primary-color)",
      "#e04d80": "var(--primary-color)",
      "#9f315e": "var(--primary-color)",
      "#c53469": "var(--primary-color)",
      "#ef8fb4": "var(--primary-color)",
      "#E85FB2": "var(--primary-color)",
      "#8C2968": "var(--primary-color)",
      "#7F0047": "var(--primary-color)",
      "#891455": "var(--primary-color)",
      "#AC5A88": "var(--primary-color)",
      "#C995B2": "var(--primary-color)",
      "#E0C2D3": "var(--primary-color)",
      "#FF7087": "var(--primary-color)",
      "#FDBB46": "var(--primary-color)",
      "#FD8946": "var(--primary-color)",
      "#e13975": "var(--primary-color)",
      "#6d1d50": "var(--secondary-color)",
      "#f7ccdd": "var(--primary-light)",

      // Secondary color gradients
      "#090029": "var(--secondary-color)",
      "#282f39": "var(--secondary-color)",
      "#39414d": "var(--secondary-color)",
      "#717982": "var(--secondary-color)",
      "#414954": "var(--secondary-color)",
      ...colorMap,
    };

    const loadSvg = async () => {
      try {
        setIsLoading(true);
        setError(null);

        // Fetch the SVG file
        const response = await fetch(src);

        if (!response.ok) {
          throw new Error(
            `Failed to fetch SVG: ${response.status} ${response.statusText}`
          );
        }

        let svgText = await response.text();

        // Replace colors with theme variables
        Object.entries(defaultColorMap).forEach(
          ([originalColor, themeVariable]) => {
            // Use regex to replace all occurrences of the color (case insensitive)
            const regex = new RegExp(originalColor, "gi");
            svgText = svgText.replace(regex, themeVariable);
          }
        );

        // Handle specific gradient-related attributes
        svgText = svgText.replace(
          /stop-color="#e13975"/g,
          'stop-color="var(--primary-color)"'
        );
        svgText = svgText.replace(
          /stop-color="#6d1d50"/g,
          'stop-color="var(--secondary-color)"'
        );
        svgText = svgText.replace(
          /fill="#f7ccdd"/g,
          'fill="var(--primary-light)"'
        );
        svgText = svgText.replace(
          /fill="url\(#linear-gradient\)"/g,
          'fill="var(--primary-color)"'
        );

        // Extract viewBox, width and height from SVG
        const viewBoxMatch = svgText.match(/viewBox="([^"]+)"/);
        const widthMatch = svgText.match(/width="([^"]+)"/);
        const heightMatch = svgText.match(/height="([^"]+)"/);

        // Ensure SVG has proper viewBox if missing
        if (!viewBoxMatch && widthMatch && heightMatch) {
          const w = widthMatch[1].replace(/[^0-9.]/g, "");
          const h = heightMatch[1].replace(/[^0-9.]/g, "");
          svgText = svgText.replace(/<svg/, `<svg viewBox="0 0 ${w} ${h}"`);
        }

        // Always make SVG responsive by setting width and height to 100%
        // and using the container div to control actual dimensions
        svgText = svgText
          .replace(/width="([^"]+)"/, 'width="100%"')
          .replace(/height="([^"]+)"/, 'height="100%"');

        // Add preserveAspectRatio attribute if not present
        if (!svgText.includes("preserveAspectRatio")) {
          svgText = svgText.replace(
            /<svg/,
            '<svg preserveAspectRatio="xMidYMid meet"'
          );
        }

        setSvgContent(svgText);
        setIsLoading(false);
      } catch (error) {
        console.error("Error loading SVG:", error);
        setError(error.message);
        setIsLoading(false);
      }
    };

    loadSvg();
  }, [src, colorMap]);

  // Determine the style for the container
  const containerStyle = {
    display: "inline-block",
    overflow: "hidden",
    verticalAlign: "middle",
  };

  // Add width and height if provided
  if (width) containerStyle.width = width;
  if (height) containerStyle.height = height;

  // Show fallback while loading
  if (isLoading) {
    return (
      <div
        className={`${className || ""} bg-gray-100 animate-pulse`}
        style={containerStyle}
        aria-label={`Loading ${alt || "SVG"}`}
      ></div>
    );
  }

  // Show error state
  if (error) {
    console.warn(`ThemeSvg error: ${error}`);
    return (
      <div
        className={`${
          className || ""
        } bg-gray-100 flex items-center justify-center text-gray-500`}
        style={containerStyle}
      >
        <span>SVG Error</span>
      </div>
    );
  }

  // If we have SVG content, render it directly
  if (svgContent) {
    return (
      <div
        className={className || ""}
        style={containerStyle}
        dangerouslySetInnerHTML={{ __html: svgContent }}
        aria-label={alt || "SVG Image"}
        role="img"
      />
    );
  }

  // Fallback if something went wrong
  return null;
};

export default ThemeSvg;
